@@ -16,29 +16,28 @@ import re
 import sys
 import unittest
 
+import ddt
 import fixtures
 import keystoneauth1.exceptions as ks_exc
 from keystoneauth1.exceptions import DiscoveryFailure
+from keystoneauth1.identity.generic.password import Password as ks_password
 from keystoneauth1 import session
 import mock
-import pkg_resources
 import requests_mock
-import requests
 from six import moves
 from testtools import matchers
 
 import cinderclient
 from cinderclient import api_versions
+from cinderclient.contrib import noauth
 from cinderclient import exceptions
-from cinderclient import auth_plugin
 from cinderclient import shell
 from cinderclient.tests.unit import fake_actions_module
-from cinderclient.tests.unit.test_auth_plugins import mock_http_request
-from cinderclient.tests.unit.test_auth_plugins import requested_headers
 from cinderclient.tests.unit.fixture_data import keystone_client
 from cinderclient.tests.unit import utils
 
 
+@ddt.ddt
 class ShellTest(utils.TestCase):
 
     FAKE_ENV = {
@@ -76,6 +75,41 @@ class ShellTest(utils.TestCase):
 
         return out
 
+    def test_default_auth_env(self):
+        _shell = shell.OpenStackCinderShell()
+        args, __ = _shell.get_base_parser().parse_known_args([])
+        self.assertEqual('', args.os_auth_type)
+
+    def test_auth_type_env(self):
+        self.make_env(exclude='OS_PASSWORD',
+                      include={'OS_AUTH_SYSTEM': 'non existent auth',
+                               'OS_AUTH_TYPE': 'noauth'})
+        _shell = shell.OpenStackCinderShell()
+        args, __ = _shell.get_base_parser().parse_known_args([])
+        self.assertEqual('noauth', args.os_auth_type)
+
+    def test_auth_system_env(self):
+        self.make_env(exclude='OS_PASSWORD',
+                      include={'OS_AUTH_SYSTEM': 'noauth'})
+        _shell = shell.OpenStackCinderShell()
+        args, __ = _shell.get_base_parser().parse_known_args([])
+        self.assertEqual('noauth', args.os_auth_type)
+
+    @mock.patch.object(cinderclient.shell.OpenStackCinderShell,
+                       '_get_keystone_session')
+    @mock.patch.object(cinderclient.client.SessionClient, 'authenticate',
+                       side_effect=RuntimeError())
+    def test_password_auth_type(self, mock_authenticate,
+                                mock_get_session):
+        self.make_env(include={'OS_AUTH_TYPE': 'password'})
+        _shell = shell.OpenStackCinderShell()
+
+        # We crash the command after Client instantiation because this test
+        # focuses only keystoneauth1 indentity cli opts parsing.
+        self.assertRaises(RuntimeError, _shell.main, ['list'])
+        self.assertIsInstance(_shell.cs.client.session.auth,
+                              ks_password)
+
     def test_help_unknown_command(self):
         self.assertRaises(exceptions.CommandError, self.shell, 'help foofoo')
 
@@ -100,6 +134,15 @@ class ShellTest(utils.TestCase):
             self.assertThat(help_text,
                             matchers.MatchesRegex(r, re.DOTALL | re.MULTILINE))
 
+    @ddt.data('backup-create --help', '--help backup-create')
+    def test_dash_dash_help_on_subcommand(self, cmd):
+        required = ['.*?^Creates a volume backup.']
+        help_text = self.shell(cmd)
+
+        for r in required:
+            self.assertThat(help_text,
+                            matchers.MatchesRegex(r, re.DOTALL | re.MULTILINE))
+
     def register_keystone_auth_fixture(self, mocker, url):
         mocker.register_uri('GET', url,
                             text=keystone_client.keystone_request_callback)
@@ -173,49 +216,19 @@ class ShellTest(utils.TestCase):
             tenant_name=self.FAKE_ENV['OS_TENANT_NAME'],
             username=self.FAKE_ENV['OS_USERNAME'])
 
-    @mock.patch.object(requests, "request")
-    @mock.patch.object(pkg_resources, "iter_entry_points")
-    def test_auth_system_not_keystone(self, mock_iter_entry_points,
-                                      mock_request):
-        """Test that we can authenticate using the auth plugin system."""
-        non_keystone_auth_url = "http://non-keystone-url.com/v2.0"
-
-        class MockEntrypoint(pkg_resources.EntryPoint):
-            def load(self):
-                return FakePlugin
-
-        class FakePlugin(auth_plugin.BaseAuthPlugin):
-            def authenticate(self, cls, auth_url):
-                cls._authenticate(auth_url, {"fake": "me"})
-
-            def get_auth_url(self):
-                return non_keystone_auth_url
-
-        mock_iter_entry_points.side_effect = lambda _t: [
-            MockEntrypoint("fake", "fake", ["FakePlugin"])]
-
-        mock_request.side_effect = mock_http_request()
-
-        # Tell the shell we wish to use our 'fake' auth instead of keystone
-        # and the auth plugin will provide the auth url
-        self.make_env(exclude="OS_AUTH_URL",
-                      include={'OS_AUTH_SYSTEM': 'fake'})
-        # This should fail as we have not setup a mock response for 'list',
-        # however auth should have been called
+    @requests_mock.Mocker()
+    def test_noauth_plugin(self, mocker):
+        os_auth_url = "http://example.com/v2"
+        mocker.register_uri('GET',
+                            "%s/admin/volumes/detail"
+                            % os_auth_url, text='{"volumes": []}')
         _shell = shell.OpenStackCinderShell()
-        self.assertRaises(KeyError, _shell.main, ['list'])
-
-        headers = requested_headers(_shell.cs)
-        token_url = _shell.cs.client.auth_url + "/tokens"
-        self.assertEqual(non_keystone_auth_url + "/tokens", token_url)
-
-        mock_request.assert_any_call(
-            "POST",
-            token_url,
-            headers=headers,
-            data='{"fake": "me"}',
-            allow_redirects=True,
-            **self.TEST_REQUEST_BASE)
+        args = ['--os-endpoint', os_auth_url,
+                '--os-auth-type', 'noauth', '--os-user-id',
+                'admin', '--os-project-id', 'admin', 'list']
+        _shell.main(args)
+        self.assertIsInstance(_shell.cs.client.session.auth,
+                              noauth.CinderNoAuthPlugin)
 
     @mock.patch.object(cinderclient.client.HTTPClient, 'authenticate',
                        side_effect=exceptions.Unauthorized('No'))
